////////////////////////////////////////////////////////////////////// 
// extras/pedstats_pdf.cpp 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday June 11, 2003
// 
 
// This file was written by Jan Wigginton 


#include "pedstats.h"
#include "pedstats_pdf.h"

// externally defined functions and variables
//

extern String pedName;
extern Pedigree ped;
extern int affection_filter, trait_filter, covariate_filter;
extern bool pairs;
int call = 0;

void GraphFamilyStatistics(PDF & pdf)
   {
   Vector familyMembers;
   Vector familyFounders;
   Vector familyGenerations;
   Vector familyNonFounders;

   int individuals = 0;

   int min_gens, max_gens ;
   int min_founders, max_founders ;
   int min_nonfounders, max_nonfounders;
   int min_members, max_members;

   for (int f = 0; f < ped.familyCount; f++)
      {
      if (f == 0)
         {
         min_founders = max_founders = ped.families[f]->founders;
         min_nonfounders = max_nonfounders = ped.families[f]->count - ped.families[f]->founders;
         min_members = max_members = ped.families[f]->count;
         }
      familyFounders.Push(ped.families[f]->founders);
      max_founders = max(max_founders, ped.families[f]->founders);
      min_founders = min(min_founders, ped.families[f]->founders);

      int non_founders = ped.families[f]->count  - ped.families[f]->founders;
      familyNonFounders.Push(non_founders);
      min_nonfounders = min(min_nonfounders, non_founders);
      max_nonfounders = max(max_nonfounders, non_founders);

      familyMembers.Push(ped.families[f]->count);
      max_members = max(max_members, ped.families[f]->count);
      min_members = min(min_members, ped.families[f]->count);

      int gens = 0;
      for (int i = ped.families[f]->first; i <= ped.families[f]->last; i++)
         gens = max(Generation(ped.persons[i]) + 1, gens);
      familyGenerations.Push(gens);

      if (f == 0)
         min_gens = max_gens = gens;
      else
        {
        max_gens = max(max_gens, gens);
        min_gens = min(min_gens, gens);
        }

      individuals += ped.families[f]->count;
      }

    pdf.page.OpenPage();

    // settings for family member charts
    PDFHistogram family_chart;
    family_chart.useLegend = false;
    family_chart.yAxis.useDiscreteValues = true;

    family_chart.yAxis.label = "Number of pedigrees";
    family_chart.Dimension(1, ped.familyCount);

    // overall
    family_chart.SetDataValues(0, familyMembers);
    family_chart.SetSeriesColor(0, 255, 0, 0);
    family_chart.SetCategoricalBars(true, min_members, max_members);

    family_chart.title = "Individuals";
    family_chart.xAxis.label = "Number of family members";

    family_chart.DrawInUpperLeft(pdf);

    // founders
    family_chart.SetDataValues(0, familyFounders);
    family_chart.SetCategoricalBars(true, min_founders, max_founders);
    family_chart.SetSeriesColor(0, 0, 255, 255);

    family_chart.title = "Founders";
    family_chart.xAxis.label = "Number of founders";

    family_chart.DrawInLowerRight(pdf);

    // non-founders
    family_chart.SetDataValues(0, familyNonFounders);
    family_chart.SetCategoricalBars(true, min_nonfounders, max_nonfounders);
    family_chart.SetSeriesColor(0, 0, 255, 0);

    family_chart.title = "Non-founders";
    family_chart.xAxis.label = "Number of non-founders";

    family_chart.DrawInLowerLeft(pdf);

    // number of generations
    family_chart.SetDataValues(0, familyGenerations);
    family_chart.SetCategoricalBars(true, min_gens, max_gens);
    family_chart.SetSeriesColor(0, 0, 0, 255);

    family_chart.title = "Generations";
    family_chart.xAxis.label = "Number of generations";

    family_chart.DrawInUpperRight(pdf);

    // make page header
    int ct = ped.familyCount;
    String sub_title;
    sub_title.printf("( %d individual%s in %d famil%s )",
                     individuals, individuals != 1 ? "s" : "",
                     ct, ct != 1 ? "ies" : "y");

    MakePageHeader(pdf, "Summary of Family Structure", sub_title);

    pdf.page.ClosePage();
   }

void GraphTraitStatistics(PDF & pdf)
   {
   if (Person::traitCount == 0) return;

   for (int trait = 0; trait < Person::traitCount; trait++)
      {
      call = 0;
      GraphTrait(pdf, trait_filter = trait);
      if (pairs)  GraphPairVars(pdf, false);
      }
   }

void GraphTrait(PDF & pdf, int t)
   {
   int    tcount = 0;

   Vector allTraitValues;
   Vector founderTraitValues, nonFounderTraitValues;
   Vector femaleTraitValues, maleTraitValues;

   double trait_value;

   for (int i = 0; i < ped.count; i++)
      {
      if ( !ped[i].isPhenotyped(t) ) continue;

      tcount++;
      trait_value = ped[i].traits[t];

      allTraitValues.Push(trait_value);

      if (ped[i].isFounder())
          founderTraitValues.Push(trait_value);
      else
          nonFounderTraitValues.Push(trait_value);

      if (ped[i].sex == SEX_MALE)
          maleTraitValues.Push(trait_value);
      else if (ped[i].sex == SEX_FEMALE)
          femaleTraitValues.Push(trait_value);
      }

    pdf.page.OpenPage();

    PDFHistogram trait_value_chart;
    trait_value_chart.xAxis.label = "Trait value";
    trait_value_chart.yAxis.label = "Individuals";
    trait_value_chart.stacked = false;

    GraphVarOverall(pdf, trait_value_chart, allTraitValues);
    GraphVarBySex(pdf, trait_value_chart, femaleTraitValues, maleTraitValues, true);
    GraphVarByFounder(pdf, trait_value_chart, founderTraitValues, nonFounderTraitValues, true);
    GraphVarByPairs(pdf, false, "Sib", "sib", isSib, true);

    String title;
    title.printf("Distribution of Values for %.20s Trait", (const char *) ped.traitNames[t]);

    String sub_title;
    if (tcount > 0)
       sub_title.printf( "( Mean: %.2f Variance: %.2f )",
        allTraitValues.Average(), (tcount <= 1) ? 0.0 : allTraitValues.Var());

    MakePageHeader(pdf, title, sub_title);

    pdf.page.ClosePage();

   }

void GraphCovariateStatistics(PDF & pdf)
   {
   if (Person::covariateCount == 0) return;

   for (int covariate = 0; covariate < Person::covariateCount; covariate++)
     {
     call = 0;
     GraphCovariate(pdf, covariate_filter = covariate);
     if (pairs) GraphPairVars(pdf, true);
     }
   }

void GraphCovariate(PDF & pdf, int c)
   {
   Vector allCovValues;
   Vector founderCovValues, nonFounderCovValues;
   Vector maleCovValues, femaleCovValues;

   int    tcount = 0;
   double cov_value;

   for (int i = 0; i < ped.count; i++)
      {
      if ( !ped[i].isControlled(c) ) continue;
      tcount++;

      cov_value = ped[i].covariates[c];

      allCovValues.Push(cov_value);

      if (ped[i].isFounder())
         founderCovValues.Push(cov_value);
      else
         nonFounderCovValues.Push(cov_value);

      if (ped[i].sex == SEX_MALE)
         maleCovValues.Push(cov_value);
      else if (ped[i].sex == SEX_FEMALE)
         femaleCovValues.Push(cov_value);
      }

   pdf.page.OpenPage();

   PDFHistogram covariate_chart;
   covariate_chart.xAxis.label = "Covariate value";
   covariate_chart.yAxis.label = "Individuals";
   covariate_chart.stacked = false;

   GraphVarOverall(pdf, covariate_chart, allCovValues);
   GraphVarBySex(pdf, covariate_chart, femaleCovValues, maleCovValues, true);
   GraphVarByFounder(pdf, covariate_chart, founderCovValues, nonFounderCovValues, true);
   GraphVarByPairs(pdf, true,"Sib", "sib", isSib, true);

   // page header
   String sub_title = "", title;

   title.printf("Distribution of Values for: %-15.15s", (const char *) ped.covariateNames[c]);

   if (tcount > 0)
     sub_title.printf("( Mean: %.2f Variance: %.2f )", allCovValues.Average(),
                     tcount <= 1 ? 0.0 : allCovValues.Var());

   MakePageHeader(pdf, title, sub_title);

   pdf.page.ClosePage();
   }

void GraphVarOverall(PDF & pdf, PDFHistogram & chart, Vector & values)
   {
   chart.Dimension(1, values.dim);
   chart.SetDataValues(0, values);

   chart.SetSeriesColor(0, 0, 0, 255);
   if (!chart.xAxis.useDiscreteValues) chart.SetBarSpacing(0.0);
   chart.narrowBars = true;
   chart.useLegend = false;

   chart.title = "Overall distribution";

   chart.DrawInUpperLeft(pdf);
   chart.narrowBars = false;
   }

void GraphVarBySex(PDF & pdf, PDFHistogram & chart, Vector & v_female, Vector & v_male,
                   bool do_subtitle)
   {
   int max_dim = max(v_female.dim, v_male.dim);
   chart.Dimension(2, max_dim);
   chart.SetDataValues(0, v_female);
   chart.SetDataValues(1, v_male);

   chart.SetSeriesColor(0, 0, 0, 255);
   chart.SetSeriesColor(1, 0, 255, 0);
   if (!chart.xAxis.useDiscreteValues) chart.SetBarSpacing(0.1);
   chart.useLegend = true;

   chart.title = "Distribution by sex ";
   if (do_subtitle)
      {
      int male_ct = v_male.SafeCount();
      int female_ct = v_female.SafeCount();

      chart.subTitle.printf("( %d %s, %d %s )", male_ct, (male_ct != 1) ?  "males" : "male",
                                female_ct, (female_ct != 1) ? "females": "female" );
      }

   chart.SetSeriesLabel(0, "Females");
   chart.SetSeriesLabel(1, "Males");

   chart.DrawInUpperRight(pdf);
   chart.subTitle = "";
   }

void GraphVarByFounder(PDF & pdf, PDFHistogram & chart, Vector & v_founder, Vector & v_nonfounder,
                        bool do_subtitle)
   {
   int max_dim = max(v_founder.dim, v_nonfounder.dim);
   chart.Dimension(2, max_dim);

   chart.SetDataValues(0, v_founder);
   chart.SetDataValues(1, v_nonfounder);

   if (!chart.xAxis.useDiscreteValues) chart.SetBarSpacing(0.1);

   chart.title = "Distribution by founder status";

   if (do_subtitle)
      {
      int founder_ct = v_founder.SafeCount();
      int nonfounder_ct = v_nonfounder.SafeCount();


      chart.subTitle.printf("( %d %s, %d %s )",
                      founder_ct,
                      founder_ct != 1 ?  "founders" : "founder",
                      nonfounder_ct,
                      nonfounder_ct != 1 ? "non-founders": "non-founder");
      }

   chart.SetSeriesLabel(0, "Founder");
   chart.SetSeriesLabel(1, "Non-Founder");

   chart.SetSeriesColor(0, 0, 0 , 255);
   chart.SetSeriesColor(1, 0, 255, 0);

   chart.DrawInLowerRight(pdf);
   chart.subTitle = "";
   }

void GraphVarByPairs(PDF & pdf, bool is_covariate, const char *x_tag, const char* y_tag,
   bool (*relationship_filter)(Person *, Person *), bool is_ordered, const char * alt_title)
   {
   PDFLineChart chart;

   chart.yAxis.frameData = chart.xAxis.frameData = true;
   chart.symmetricAxes = true;
   chart.title = alt_title;

   String lc_tag = y_tag;
   lc_tag.ToLower();

   if (chart.title == "")
      chart.title.printf("%s-%s plot", x_tag, (const char *) lc_tag);

   chart.xAxis.label.printf("%s %s value", x_tag, is_covariate ? "covariate" : "trait");

   if (is_ordered)
      chart.yAxis.label.printf("Co-%s %s value", y_tag, is_covariate ? "covariate" : "trait");
   else
      chart.yAxis.label.printf("%s %s value", y_tag, is_covariate ? "covariate" : "trait");

   int pairs = 0;

   double r_pair = is_covariate ?
      GetPairData(FilterOnCovariate, relationship_filter, chart, pairs, is_covariate, is_ordered) :
      GetPairData(FilterOnTrait, relationship_filter, chart, pairs, is_covariate, is_ordered);

   int actual_pairs = is_ordered ? pairs / 2 : pairs;

   String temp;
   if (actual_pairs > 0)
      {
      temp.printf("( %d phenotyped %s ",actual_pairs, actual_pairs != 1 ? "pairs" : "pair");

      String add;
      r_pair != _NAN_ ? add.printf(": r = %5.2lf )", r_pair) : add.printf(" )");
      temp += add;
      }
   else
      temp.printf("( No phenotyped pairs )");
   chart.subTitle += temp;

   chart.ShowLine(0, false);
   chart.ShowMarker(0,true);

   if (pairs <= 4)
     chart.SetMarkerRadius(0, 5.0);
   else if (pairs < 20)
      chart.SetMarkerRadius(0, 4.0);
   else if (pairs < 40)
     chart.SetMarkerRadius(0, 2.0);

   chart.useLegend = false;

   if (call == 0)
      chart.DrawInLowerLeft(pdf);
   else
      DrawArea(pdf, &chart, call);

   call++;
   }

double GetPairData(bool (*filter)(Person &, Person &), bool (*relationship_filter)(Person *, Person *),
   PDFLineChart & chart, int & pairs, bool is_covariate, bool is_ordered)
   {
   Vector x_vals;
   Matrix y_vals;

   y_vals.Dimension(1, 0);
   x_vals.Dimension(0);

   double val1, val2;

   for (int f = 0; f < ped.familyCount; f++)
      for (int i = ped.families[f]->first; i <= ped.families[f]->last; i++)
         for (int j = ped.families[f]->first; j <= ped.families[f]->last; j++)
            {
            if (j == i) continue;
            if (filter == NULL || filter(ped[i], ped[j]))
               {
               if (relationship_filter(&ped[i], &ped[j]))
                  {
                  pairs++;

                  if (is_covariate)
                     {
                     val1 = ped[i].covariates[covariate_filter];
                     val2 = ped[j].covariates[covariate_filter];
                     }
                  else
                     {
                     val1 = ped[i].traits[trait_filter];
                     val2 = ped[j].traits[trait_filter];
                     }
                  x_vals.Push(val1);
                  y_vals[0].Push(val2);
                  }
               }
            }

   x_vals.Dimension(pairs);
   y_vals.Dimension(1, pairs);
   chart.Dimension(1, pairs);

   chart.SetDataValues(x_vals, y_vals);

   double r_pair = _NAN_;

   if (pairs > 1)
      {
      double s_xx = x_vals.SumSquares() - pairs * x_vals.Average() * x_vals.Average();
      double s_yy = y_vals[0].SumSquares() - pairs * y_vals[0].Average() * y_vals[0].Average();
      double s_xy = x_vals.InnerProduct(y_vals[0]) - pairs * y_vals[0].Average() * x_vals.Average();
      double denom = s_xx * s_yy;
      if (denom > 0.0) r_pair =  s_xy / sqrt(denom);
      }

   return r_pair;
   }

void GraphMarkerStatistics(PDF & pdf)
   {
   if (Person::markerCount == 0) return;

   for (int m = 0; m < Person::markerCount; m++)
      GraphMarker(pdf, m);
   }

void GraphMarker(PDF & pdf, int m)
   {
   int mGenotypes = 0, mPossible = 0;
   int lastAllele = 0, firstAllele = 100000;
   int ct_males = 0, ct_females = 0;
   int ct_founders = 0, ct_nonfounders = 0;

   Vector overallAlleles;
   Vector founderAlleles, nonFounderAlleles;
   Vector maleAlleles, femaleAlleles;
   Vector hetCounts, homCounts;

   for (int i = 0; i < ped.count; i++)
      {
      mPossible  ++;
      if ( !ped[i].isGenotyped(m) ) continue;
      mGenotypes++;

      int m0 = ped[i].markers[m][0];
      int m1 = ped[i].markers[m][1];

      overallAlleles.Push(m0);
      overallAlleles.Push(m1);

      if (ped[i].isFounder())
         {
         ct_founders++;
         founderAlleles.Push(m0);
         founderAlleles.Push(m1);
         }
      else
         {
         ct_nonfounders++;
         nonFounderAlleles.Push(m0);
         nonFounderAlleles.Push(m1);
         }

      if (ped[i].sex == SEX_MALE)
         {
         ct_males++;
         maleAlleles.Push(m0);
         maleAlleles.Push(m1);
         }
      else if (ped[i].sex == SEX_FEMALE)
         {
         ct_females++;
         femaleAlleles.Push(m0);
         femaleAlleles.Push(m1);
         }

      firstAllele = min(min(m0, m1), firstAllele);
      lastAllele  = max(max(m0, m1), lastAllele);

      if (ped.chromosomeX && ped[i].sex == SEX_MALE) continue;

      if ( ped[i].markers[m].isHeterozygous())
         hetCounts.Push(2.0);
      else if (ped[i].markers[m].isHomozygous())
         homCounts.Push(1.0);
      }

   pdf.page.OpenPage();

   PDFHistogram marker_chart;

   // Overall distribution
   marker_chart.stacked = false;
   marker_chart.SetUseSeriesPercentages(true);
   marker_chart.SetCategoricalBars(true, firstAllele, lastAllele);
   marker_chart.yAxis.maxMax = 1.1;

   marker_chart.xAxis.label = "Allele number";
   marker_chart.yAxis.label = "Allele frequency";

   GraphVarOverall(pdf, marker_chart, overallAlleles);

   // By sex distribution
   marker_chart.subTitle.printf("( %d %s, %d %s )",
                                 ct_males, (ct_males != 1) ? "males" : "male",
                                 ct_females, (ct_females != 1) ? "females" : "female");
   GraphVarBySex(pdf, marker_chart, femaleAlleles, maleAlleles, false);

   // By founder status distribution
   marker_chart.subTitle.printf("( %d %s, %d %s )", ct_founders,
                                   ct_founders != 1 ? "founders" : "founder",
                                   ct_nonfounders,
                                   ct_nonfounders != 1 ? "non-founders" : "non-founder");
   GraphVarByFounder(pdf, marker_chart, founderAlleles, nonFounderAlleles, false);

   // Informativeness chart
   marker_chart.Dimension(2, max(hetCounts.dim, homCounts.dim));
   marker_chart.SetDataValues(0, homCounts);
   marker_chart.SetDataValues(1, hetCounts);

   marker_chart.SetCategoricalBars(true, 1.0, 2.0);
   marker_chart.SetUseTotalPercentages(true);
   marker_chart.stacked = true;

   marker_chart.xAxis.SetStringTickLabels(true);
   marker_chart.xAxis.SetTickLabel(0, "Homozygote");
   marker_chart.xAxis.SetTickLabel(1, "Heterozygote");
   marker_chart.xAxis.useTicks = true;

   marker_chart.xAxis.SetMin(0.5);
   marker_chart.xAxis.SetMax(2.5);

   marker_chart.SetSeriesColor(0, 255, 0, 0);
   marker_chart.SetSeriesColor(1, 0, 255, 0);

   marker_chart.title = "Informativeness";
   marker_chart.useLegend = false;

   if (ped.chromosomeX) marker_chart.subTitle = "( X-linked marker: Among females )";
   marker_chart.yAxis.label = "Percentage of population";
   marker_chart.xAxis.label = "";

   marker_chart.DrawInLowerLeft(pdf);
   marker_chart.subTitle = "";

   String title, sub_title;
   title.printf("Marker Allele Frequencies for: %-15.15s",   (const char *) Person::markerNames[m]);

   sub_title.printf("( %d of %d %s genotyped )", mGenotypes, mPossible,
                      mPossible != 1 ? "individuals" : "individual");

   MakePageHeader(pdf, title, sub_title);

   pdf.page.ClosePage();
   }

void GraphAffectionStatistics(PDF & pdf)
   {
   for (affection_filter = 0; affection_filter < ped.affectionCount; affection_filter++)
      {
      call = 0;
      GraphAffection(pdf, affection_filter);
      if (pairs) GraphPairAffections(pdf);
      }
   }

void GraphAffection(PDF & pdf, int index)
   {
   if (Person::affectionCount == 0) return;

   Vector nonFounderAff, founderAff;
   Vector maleAff, femaleAff;
   Vector overallAff;

   int    tcount = 0, tpossible = 0;

   for (int i = 0; i < ped.count; i++)
      {
      tpossible  ++;
      if ( !ped[i].isDiagnosed(index) ) continue;
      tcount ++;

      double aff_stat = ped[i].affections[index];

      overallAff.Push(aff_stat);

      if (ped[i].isFounder())
         founderAff.Push(aff_stat);
      else
         nonFounderAff.Push(aff_stat);

      if (ped[i].sex == SEX_MALE)
         maleAff.Push(aff_stat);
      else if (ped[i].sex == SEX_FEMALE)
         femaleAff.Push(aff_stat);
      }

   pdf.page.OpenPage();

   PDFHistogram affection_chart;

   affection_chart.stacked = false;
   affection_chart.SetCategoricalBars(true, 1.0, 2.0);
   affection_chart.yAxis.useDiscreteValues = true;
   affection_chart.xAxis.SetStringTickLabels(true);
   affection_chart.xAxis.SetTickLabel(2, "");
   affection_chart.xAxis.SetTickLabel(1, "Affected");
   affection_chart.xAxis.SetTickLabel(0, "Unaffected");

   affection_chart.yAxis.label = "Individuals";

   GraphVarOverall(pdf, affection_chart, overallAff);
   GraphVarBySex(pdf, affection_chart, femaleAff, maleAff, true);
   GraphVarByFounder(pdf, affection_chart, founderAff, nonFounderAff, true);
   GraphPairAffection(pdf, affection_chart, isSib, "Sib", index);

   String sub_title, title;
   title.printf("Affection Status Statistics for: %-15.15s",
                  (const char *) Person::affectionNames[index]);

   sub_title.printf("( %d of %d %s diagnosed )",tcount, tpossible,
                    tpossible != 1 ? "individuals" : "individual");

   MakePageHeader(pdf, (const char *) title, (const char *) sub_title);

   pdf.page.ClosePage();
   }

void GraphPairAffection(PDF & pdf, PDFHistogram & chart, bool (*relationship_filter)(Person *, Person *),
  const char *tag, int index)
   {
   Vector discordant;
   Vector concordantAff;
   Vector concordantUnaff;

   int pairs = GetPairAffectionData(discordant, concordantAff, concordantUnaff, relationship_filter, index);

   int max_dim = max(discordant.dim, max(concordantAff.dim, concordantUnaff.dim));
   chart.Dimension(3, max_dim);

   chart.SetDataValues(0, concordantUnaff);
   chart.SetDataValues(1, discordant);
   chart.SetDataValues(2, concordantAff);

   chart.stacked = true;
   chart.SetCategoricalBars(true, 1.0, 3.0);
   chart.xAxis.SetStringTickLabels(true);
   if (call > 0) chart.xAxis.useAlternatingLabels = true;

   chart.useLegend = false;

   chart.xAxis.SetTickLabel(0, "Unaffected");
   chart.xAxis.SetTickLabel(1, "Discordant");
   chart.xAxis.SetTickLabel(2, "Affected");
   chart.xAxis.label = "";

   if (call == 0)
      chart.title = "Distribution of sib pairs";
   else
      chart.title.printf("%s pairs", tag);

   if (pairs > 0)
      chart.subTitle.printf("( %d diagnosed %s )", pairs, pairs != 1 ? "pairs" : "pair");
   else
     chart.subTitle.printf("( No diagnosed pairs )");

   chart.yAxis.label.printf("%s Pairs", tag);

   chart.SetSeriesLabel(2, "Affected");
   chart.SetSeriesLabel(1, "Discordant");
   chart.SetSeriesLabel(0, "Unaffected");

   chart.SetSeriesColor(0, 0, 255, 0);
   chart.SetSeriesColor(1, 255, 255, 0);
   chart.SetSeriesColor(2, 255, 0, 0);

   if (call == 0)
      chart.DrawInLowerLeft(pdf);
   else
      DrawArea(pdf, &chart, call);

   chart.xAxis.useTicks = true;
   chart.xAxis.SetNumericTickLabels(true);

   call++;
   }

int GetPairAffectionData(Vector & d_pairs, Vector & ca_pairs, Vector & cu_pairs,
   bool (*relationship_filter)(Person *, Person *),int index)
   {
   int pairs = 0;

   for (int f = 0; f < ped.familyCount; f++)
      for (int i = ped.families[f]->first; i <= ped.families[f]->last; i++)
         for (int j = i + 1; j <= ped.families[f]->last; j++)
            {
            if (j == i) continue;
            if (relationship_filter(&ped[i], &ped[j]))
               {
               int stat_1 = ped[i].affections[index];
               int stat_2 = ped[j].affections[index];

               if ( stat_1 == 1 && stat_2 == 1)
                  {
                  cu_pairs.Push(1.0);
                  pairs++;
                  }
               else if ( stat_1 == 2 && stat_2 == 2)
                  {
                  ca_pairs.Push(3.0);
                  pairs++;
                  }
               else if ( stat_1 == 2 && stat_2 == 1 || stat_1 == 1 && stat_2 == 2)
                  {
                  d_pairs.Push(2.0);
                  pairs++;
                  }
               }
            }
   return pairs;
   }

void MakePageHeader(PDF & pdf, const char *title, const char *sub_title)
   {
   pdf.page.hTextAlignment = taCenter;
   pdf.page.vTextAlignment = taMiddle;

   pdf.page.SetFontSize(16);
   pdf.page.WriteText(pdf.page.GetWidth() * 0.5, pdf.page.GetHeight() * 0.95, title);

   pdf.page.SetFontSize(10);
   pdf.page.WriteText(pdf.page.GetWidth() * 0.5, pdf.page.GetHeight() * 0.91, sub_title);
   }

void GraphPairAffections(PDF & pdf)
   {
   pdf.page.OpenPage();

   PDFHistogram chart;

   call = 1;

   GraphPairAffection(pdf, chart, isGrandParentOffspring, "Grandparent-grandchild", affection_filter);
   GraphPairAffection(pdf, chart, isParent,"Parent-child", affection_filter);
   GraphPairAffection(pdf, chart, isAvuncular, "Avuncular",affection_filter);

   GraphPairAffection(pdf, chart, isHalfSib, "Half sib", affection_filter);
   GraphPairAffection(pdf, chart, isSib, "Sib", affection_filter);
   GraphPairAffection(pdf, chart, isCousin, "Cousin", affection_filter);

   String title;
   title.printf("Relative pair distributions for %s",(const char *) Person::affectionNames[affection_filter] );

   MakePageHeader(pdf, title, "");

   pdf.page.ClosePage();

   call = 0;
   }

void GraphPairVars(PDF & pdf, bool is_covariate)
   {
   pdf.page.OpenPage();

   call = 1;

   GraphVarByPairs(pdf, is_covariate, "Grandparent", "Grandchild", isOrderedGrandParentOffspring, false);
   GraphVarByPairs(pdf, is_covariate, "Parent", "Child", isOrderedParent, false);
   GraphVarByPairs(pdf, is_covariate, "Uncle/aunt", "Nephew/niece", isOrderedAvuncular, false, "Avuncular plot");

   GraphVarByPairs(pdf, is_covariate, "Half sib", "half sib", isHalfSib, true);
   GraphVarByPairs(pdf, is_covariate, "Sib", "sib", isSib, true);
   GraphVarByPairs(pdf, is_covariate, "Cousin", "cousin", isCousin, true);

   String title;
   title.printf("Relative pair plots for %s", is_covariate ?
                                            (const char *) ped.covariateNames[covariate_filter] :
                                            (const char *) ped.traitNames[trait_filter]);
   MakePageHeader(pdf, title, "");

   pdf.page.ClosePage();

   call = 0;
   }

void DrawArea(PDF & pdf, PDFChartBasics * chart, int call)
  {
  switch (call)
     {
     case 1: DrawInArea(pdf, chart, 0, 0, 2, 3); break;
     case 2: DrawInArea(pdf, chart, 0, 1, 2, 3); break;
     case 3: DrawInArea(pdf, chart, 0, 2, 2, 3); break;
     case 4: DrawInArea(pdf, chart, 1, 0, 2, 3); break;
     case 5: DrawInArea(pdf, chart, 1, 1, 2, 3); break;
     case 6: DrawInArea(pdf, chart, 1, 2, 2, 3); break;
     }
  }

void DrawInArea(PDF & pdf, PDFChartBasics * chart, int row_to_use, int col_to_use, int rows, int cols)
   {
   double page_width = pdf.page.GetWidth();
   double page_height = pdf.page.GetHeight();
   bool legend_setting = chart->useLegend;

   double spacer = 0.05;

   double x0 = 0.99 * page_width * (double(col_to_use)/cols);
   double y0 = 0.9 * page_height * (double(row_to_use)/rows + spacer);
   double x1 = 0.99 * page_width * ((col_to_use + 1.0)/ cols );
   double y1 = 0.9 * page_height * ((row_to_use + 1.0)/rows) - spacer;

   chart->useLegend = false;

   chart->DrawInBox(pdf, x0, y0, x1, y1);
   chart->useLegend = legend_setting;
   }





 
 
