////////////////////////////////////////////////////////////////////// 
// merlin/KAC.h 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday June 11, 2003
// 
 
#ifndef __KAC_H__
#define __KAC_H__

#include "Pedigree.h"
#include "MathMatrix.h"
#include "MerlinPDF.h"

class FamilyAnalysis;

class KAC : public VectorFunc
   {
   public:
      int affectionCount, familyCount, positionCount;
      Vector * min;         // min[affection][family]
      Vector * max;         // max[affection][family]
      Matrix * scores;      // scores[affection][family][position]
      StringArray & labels; // labels for analysis positions
      StringArray pheno;    // labels for phenotypes

      KAC(Pedigree & ped, StringArray & labels);
      ~KAC();

      void LabelPhenotypes(bool all, bool pairs, bool qtl, bool dev);
      void Prepare(int affections, int families);
      void Set(int affection, int family, double lo, double hi, Vector & z);
      void Analyse(MerlinPDF & pdf);

      void SkipFamily(int serial);

      // These functions manage [merlin.lod] files for storing individual
      // scores for each family
      void OpenPerFamilyFiles();
      void ClosePerFamilyFiles();
      void OutputPerFamilyScores(int affection, int position, double delta);

   private:
      double Evaluate(Vector & parameters);

      int    currentAffection, currentPosition;
      double minimum, range;

      double MakeDelta(double value);
      void   OutputScores(const char * pos, double z, double d, double chisq);

      // Output file for individual scores
      FILE * perfamily;

      // Reference to original pedigree
      Pedigree & ped;
   };

#endif
 
 
