////////////////////////////////////////////////////////////////////// 
// merlin/Merlin.cpp 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday June 11, 2003
// 
 
#include "MerlinParameters.h"
#include "MerlinSimulator.h"
#include "MerlinFamily.h"
#include "MerlinSort.h"
#include "Manners.h"
#include "Error.h"

#include "LongInt.h"

void ShowBanner();

int main(int argc, char * argv[])
   {
   ShowBanner();

   SetupCrashHandlers();

   MerlinParameters pl;

   pl.Read(argc, argv);
   pl.Status();

#ifdef __CHROMOSOME_X__
   PedigreeGlobals::chromosomeX = true;
#endif

   Pedigree ped;

   ped.Prepare(pl.dataFile);
   ped.Load(pl.pedigreeFile);
   SortFamilies(ped);
   ped.LoadMarkerMap(pl.mapFile);
   ped.LoadAlleleFrequencies(pl.freqFile);
   ped.EstimateFrequencies(pl.alleleFrequencies);
   ped.LumpAlleles(0.0);

   if (pl.trimPedigree)
      ped.Trim();

   if (ped.markerCount < 1)
      error("The data set includes no genetic markers\n\n");

   int  next_chromosome  = -1;
   bool many_chromosomes = false;

   FamilyAnalysis engine(ped);

   engine.SetupGlobals();
   engine.SetupFiles();

   do {
      next_chromosome = engine.SetupMap(next_chromosome);

      many_chromosomes = many_chromosomes || (next_chromosome != 0);

      if (many_chromosomes)
         printf("\nAnalysing Chromosome %d\n\n",
                ped.GetMarkerInfo(engine.markers[0])->chromosome);

      if (pl.simulateNull)
         SimulateNullData(ped, engine.markers);

      for (int i = 0; i < ped.familyCount; i++)
         if (engine.SelectFamily(ped.families[i]))
            engine.Analyse();

      printf("\n");
      engine.ShowLikelihood();
      engine.ShowLODs();
      engine.ShowInformation();
   } while (next_chromosome);

   if (pl.saveReplicate)
      {
      printf("Saving simulated pedigree ...\n\n");
      ped.WriteDataFile("merlin-replicate.dat");
      ped.WritePedigreeFile("merlin-replicate.ped");
      ped.WriteMapFile("merlin-replicate.map");
      ped.WriteFreqFile("merlin-replicate.freq");
      }

   printf("\n");

   engine.CloseFiles();

   if (pl.writeFrequencyFile)
      {
      ped.WriteFreqFile("merlin.freq");
      printf("Allele frequencies written to file [merlin.freq]\n");
      }

   printf("\n");
   }

void ShowBanner()
   {
#ifndef VERSION
   printf("MERLIN - (c) 2000-2003 Goncalo Abecasis");
#else
   printf("MERLIN " VERSION " - (c) 2000-2003 Goncalo Abecasis");
#endif

#ifdef __HUGE_SWAP__
   printf("\nModifications: ");
#else
#ifdef __USE_LONG_INT
   printf("\nModifications: ");
#else
#ifdef __CHROMOSOME_X__
   printf("\nModifications: ");
#endif
#endif
#endif

#ifdef __HUGE_SWAP__
   printf("HUGE_SWAP ");
#endif

#ifdef __USE_LONG_INT
   printf("64-ALLELES ");
#endif

#ifdef __CHROMOSOME_X__
   printf("CHROMOSOME-X ");                       
#endif

   printf("\n");
   }


 
 
