////////////////////////////////////////////////////////////////////// 
// merlin/MerlinKinship.h 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday June 11, 2003
// 
 
#ifndef __MERLIN_KINSHIP_H__
#define __MERLIN_KINSHIP_H__

#include "MathMatrix.h"
#include "Pedigree.h"
#include "Mantra.h"
#include "Tree.h"

#include <stdio.h>

class MerlinKinship
   {
   public:
      MerlinKinship(Mantra & m);
      ~MerlinKinship();

      double Calculate(Tree & tree, const char * label = NULL, int position = -1);
      double Score(Tree & tree, int node, int bit, int start = 0);
      void   Output(const char * label, double scale);

      void   OpenFile(const char * filename = "merlin");
      void   CloseFile();

      void   SelectFamily(Pedigree * p, Family * f);

      void   SetFamilyCount(int families);
      void   SetPositionCount(int positions);
      void   Store(int position, double scale);

      bool   CheckFamily(int serial);
      double Retrieve(int family, int position, int index1, int index2);

      void   Discard();

   private:
      Pedigree * ped;
      Family   * family;
      Mantra   & mantra;

      // Routines for scoring kinship in trees with symmetries
      double   AlleleKinship(int allele1, int allele2);
      double   WeightedScore(Tree & tree, int node, int bit, int start,
                             double weight = 2.0);

      // Output files
      FILE     * kinfile;

      // Temporary storage
      Matrix   kinship;

      // Track uninformative meiosis in the pedigree
      IntArray uninformative;

      // Store kinship matrices for all families
      // prior to variance components analysis
      Vector * matrices;
   };

#endif

 
 
