////////////////////////////////////////////////////////////////////// 
// pdf/PDFhistogram.h 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday June 11, 2003
// 
 
/* This file written by Jan Wigginton */

#ifndef __PDFHISTOGRAM_H__
#define __PDFHISTOGRAM_H__

#include "IntArray.h"
#include "MathMatrix.h"
#include "PDF.h"
#include "PDFchartaxis.h"
#include "PDFchartlegend.h"
#include "PDFchartmarker.h"
#include "StringBasics.h"
#include "Constant.h"
#include "PDFchartbar.h"
#include "PDFchartbasics.h"

class PDFHistogram : public PDFChartBasics
   {
   public:

      bool  stacked, narrowBars;

      PDFHistogram();
      virtual ~PDFHistogram();

      // data input/chart initialization

      void ReadData(const char * infile);
      void SetDataValues(int series, const Vector & v);
      void SetDataValue(int series, int data_pt, double value);
      void SetDataValues(const Matrix & values);

      void Dimension(int num_series, int data_pts_per_series);
      void Reset();

      // chart settings/preferences

      void SetBarWidth(double width);
      void SetBarStart(double start);
      void SetBarEnd(double end);
      void SetBarSpacing(double spacing);

      void SetUseSeriesPercentages(bool use_series_pct);
      void SetUseTotalPercentages(bool use_total_pct);
      void SetCategoricalBars(bool categorical, double first_bar_category = _NAN_,
                                 double last_bar_category = _NAN_);

   private:

      // dataValues[i] holds the ith column of data. Matrix is private because
      // changes to data require recalculating the bars to update the chart.

      Matrix   dataValues;
      PDFChartBar ** bars;

      double   barStart, barEnd;
      double   barWidth, barSpacing, barSpacingPct;
      double   dataMin, dataMax;
      int      numBars;
      bool     keepBarWidth, keepBarSpacing;
      bool     useSeriesPercentages, useTotalPercentages;
      int      barSeries;

      bool tooManyBars;

      // flag for new data input

      bool dirty;

      // drawing subroutines

      virtual void DrawLegend(PDF & pdf);
      virtual void DrawBody(PDF & pdf);

      void DrawBarsStacked(PDF & pdf);
      void DrawBarsAligned(PDF & pdf);
      void DrawNoDataLabel(PDF & pdf, double x_pos);
      void DrawErrorGraph(PDF & pdf);

      // chart initialization, reinitialization for new data

      virtual bool OpenChart(PDF & pdf);
      bool CheckForNoData();

      void InitializeForNewData();
      void InitializeBars();
      void InitializeAxes();
      void Init();

      void AllocateBarMatrix(int num_series, int num_bars_per_series);
      void ResizeBarMatrix(int old_num_bars = 0, int old_num_series = 0);

      int    CalculateBarRanges(int times_tried);
      int    CalculateBarWidth(int times_tried);
      double CalculateBarCounts(const Matrix & sorted);
      void   ScaleToSeriesPct(int data_series);
      void   ScaleToTotalPct();

   };

#endif

 
 
